@@ -32,13 +32,14 @@
 #include "wds.h"
 
 #include "src/common.h"
-#include "qmimodem.h"
+#include "util.h"
 
 struct gprs_data {
 	struct qmi_device *dev;
 	struct qmi_service *nas;
 	struct qmi_service *wds;
 	unsigned int last_auto_context_id;
+	uint16_t serving_system_indication_id;
 };
 
 static bool extract_ss_info(struct qmi_result *result, int *status, int *tech)
@@ -116,7 +117,7 @@ static void get_lte_attach_param_cb(struct qmi_result *result, void *user_data)
 		ofono_info("LTE attach IP type: %hhd", iptype);
 
 	ofono_gprs_cid_activated(gprs, data->last_auto_context_id, apn);
-	g_free(apn);
+	l_free(apn);
 
 	return;
 
@@ -149,9 +150,8 @@ static void get_default_profile_cb(struct qmi_result *result, void *user_data)
 
 	data->last_auto_context_id = index;
 
-	/* Get LTE Attach Parameters */
-	if (qmi_service_send(data->wds, 0x85, NULL,
-				get_lte_attach_param_cb, gprs, NULL) > 0)
+	if (qmi_service_send(data->wds, QMI_WDS_GET_LTE_ATTACH_PARAMETERS,
+				NULL, get_lte_attach_param_cb, gprs, NULL) > 0)
 		return;
 
 error:
@@ -188,21 +188,15 @@ static void get_lte_attach_params(struct ofono_gprs* gprs)
 	 * context the modem has activated.
 	 */
 	param = qmi_param_new();
-	if (!param)
-		goto error;
 
 	/* Profile type */
 	qmi_param_append(param, 0x1, sizeof(p), &p);
 
-	/* Get default profile */
-	if (qmi_service_send(data->wds, 0x49, param,
-				get_default_profile_cb, gprs, NULL) > 0)
+	if (qmi_service_send(data->wds, QMI_WDS_GET_DEFAULT_PROFILE_NUMBER,
+				param, get_default_profile_cb, gprs, NULL) > 0)
 		return;
 
 	qmi_param_free(param);
-
-error:
-	ofono_warn("Unable to query LTE APN... will not activate context");
 }
 
 static int handle_ss_info(struct qmi_result *result, struct ofono_gprs *gprs)
@@ -290,19 +284,14 @@ static void qmi_set_attached(struct ofono_gprs *gprs, int attached,
 		action = QMI_NAS_ATTACH_ACTION_DETACH;
 
 	param = qmi_param_new_uint8(QMI_NAS_PARAM_ATTACH_ACTION, action);
-	if (!param)
-		goto error;
 
 	if (qmi_service_send(data->nas, QMI_NAS_ATTACH_DETACH, param,
-					attach_detach_cb, cbd, g_free) > 0)
+					attach_detach_cb, cbd, l_free) > 0)
 		return;
 
 	qmi_param_free(param);
-
-error:
 	CALLBACK_WITH_FAILURE(cb, cbd->data);
-
-	g_free(cbd);
+	l_free(cbd);
 }
 
 static void get_ss_info_cb(struct qmi_result *result, void *user_data)
@@ -339,13 +328,13 @@ static void qmi_attached_status(struct ofono_gprs *gprs,
 	DBG("");
 
 	cbd->user = gprs;
-	if (qmi_service_send(data->nas, QMI_NAS_GET_SS_INFO, NULL,
-					get_ss_info_cb, cbd, g_free) > 0)
+	if (qmi_service_send(data->nas, QMI_NAS_GET_SERVING_SYSTEM, NULL,
+					get_ss_info_cb, cbd, l_free) > 0)
 		return;
 
 	CALLBACK_WITH_FAILURE(cb, -1, cbd->data);
 
-	g_free(cbd);
+	l_free(cbd);
 }
 
 static void create_wds_cb(struct qmi_service *service, void *user_data)
@@ -361,16 +350,18 @@ static void create_wds_cb(struct qmi_service *service, void *user_data)
 		return;
 	}
 
-	data->wds = qmi_service_ref(service);
+	data->wds = service;
 
 	/*
 	 * First get the SS info - the modem may already be connected,
 	 * and the state-change notification may never arrive
 	 */
-	qmi_service_send(data->nas, QMI_NAS_GET_SS_INFO, NULL,
+	qmi_service_send(data->nas, QMI_NAS_GET_SERVING_SYSTEM, NULL,
 					ss_info_notify, gprs, NULL);
 
-	qmi_service_register(data->nas, QMI_NAS_SS_INFO_IND,
+	data->serving_system_indication_id =
+		qmi_service_register(data->nas,
+					QMI_NAS_SERVING_SYSTEM_INDICATION,
 					ss_info_notify, gprs, NULL);
 
 	ofono_gprs_set_cid_range(gprs, 1, 1);
@@ -391,7 +382,7 @@ static void create_nas_cb(struct qmi_service *service, void *user_data)
 		return;
 	}
 
-	data->nas = qmi_service_ref(service);
+	data->nas = service;
 
 	qmi_service_create_shared(data->dev, QMI_SERVICE_WDS,
 						create_wds_cb, gprs, NULL);
@@ -405,7 +396,7 @@ static int qmi_gprs_probe(struct ofono_gprs *gprs,
 
 	DBG("");
 
-	data = g_new0(struct gprs_data, 1);
+	data = l_new(struct gprs_data, 1);
 
 	ofono_gprs_set_data(gprs, data);
 
@@ -425,30 +416,24 @@ static void qmi_gprs_remove(struct ofono_gprs *gprs)
 
 	ofono_gprs_set_data(gprs, NULL);
 
-	qmi_service_unregister_all(data->wds);
-	qmi_service_unref(data->wds);
+	qmi_service_free(data->wds);
 
-	qmi_service_unregister_all(data->nas);
+	if (data->serving_system_indication_id) {
+		qmi_service_unregister(data->nas,
+					data->serving_system_indication_id);
+		data->serving_system_indication_id = 0;
+	}
 
-	qmi_service_unref(data->nas);
+	qmi_service_free(data->nas);
 
-	g_free(data);
+	l_free(data);
 }
 
 static const struct ofono_gprs_driver driver = {
-	.name			= "qmimodem",
 	.probe			= qmi_gprs_probe,
 	.remove			= qmi_gprs_remove,
 	.set_attached		= qmi_set_attached,
 	.attached_status	= qmi_attached_status,
 };
 
-void qmi_gprs_init(void)
-{
-	ofono_gprs_driver_register(&driver);
-}
-
-void qmi_gprs_exit(void)
-{
-	ofono_gprs_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(gprs, qmimodem, &driver)
