@@ -33,8 +33,7 @@
 
 #include "qmi.h"
 #include "pds.h"
-
-#include "qmimodem.h"
+#include "util.h"
 
 struct location_data {
 	struct qmi_service *pds;
@@ -136,19 +135,14 @@ static void qmi_location_reporting_enable(struct ofono_location_reporting *lr,
 	cbd->user = lr;
 
 	param = qmi_param_new_uint8(QMI_PDS_PARAM_AUTO_TRACKING, 0x01);
-	if (!param)
-		goto error;
 
 	if (qmi_service_send(data->pds, QMI_PDS_SET_AUTOTRACK, param,
-					autotrack_enable_cb, cbd, g_free) > 0)
+					autotrack_enable_cb, cbd, l_free) > 0)
 		return;
 
 	qmi_param_free(param);
-
-error:
 	CALLBACK_WITH_FAILURE(cb, -1, cbd->data);
-
-	g_free(cbd);
+	l_free(cbd);
 }
 
 static void autotrack_disable_cb(struct qmi_result *result, void *user_data)
@@ -181,19 +175,14 @@ static void qmi_location_reporting_disable(struct ofono_location_reporting *lr,
 	cbd->user = lr;
 
 	param = qmi_param_new_uint8(QMI_PDS_PARAM_AUTO_TRACKING, 0x00);
-	if (!param)
-		goto error;
 
 	if (qmi_service_send(data->pds, QMI_PDS_SET_AUTOTRACK, param,
-					autotrack_disable_cb, cbd, g_free) > 0)
+					autotrack_disable_cb, cbd, l_free) > 0)
 		return;
 
 	qmi_param_free(param);
-
-error:
 	CALLBACK_WITH_FAILURE(cb, cbd->data);
-
-	g_free(cbd);
+	l_free(cbd);
 }
 
 static void set_event_cb(struct qmi_result *result, void *user_data)
@@ -219,7 +208,7 @@ static void create_pds_cb(struct qmi_service *service, void *user_data)
 		return;
 	}
 
-	data->pds = qmi_service_ref(service);
+	data->pds = service;
 
 	qmi_service_register(data->pds, QMI_PDS_EVENT,
 					event_notify, lr, NULL);
@@ -228,8 +217,6 @@ static void create_pds_cb(struct qmi_service *service, void *user_data)
 					state_notify, lr, NULL);
 
 	param = qmi_param_new();
-	if (!param)
-		goto done;
 
 	qmi_param_append_uint8(param, QMI_PDS_PARAM_REPORT_NMEA, 0x01);
 	qmi_param_append_uint8(param, QMI_PDS_PARAM_REPORT_NMEA_DEBUG, 0x00);
@@ -240,7 +227,6 @@ static void create_pds_cb(struct qmi_service *service, void *user_data)
 
 	qmi_param_free(param);
 
-done:
 	ofono_location_reporting_register(lr);
 }
 
@@ -252,7 +238,7 @@ static int qmi_location_reporting_probe(struct ofono_location_reporting *lr,
 
 	DBG("");
 
-	data = g_new0(struct location_data, 1);
+	data = l_new(struct location_data, 1);
 
 	data->fd = -1;
 
@@ -271,15 +257,12 @@ static void qmi_location_reporting_remove(struct ofono_location_reporting *lr)
 
 	ofono_location_reporting_set_data(lr, NULL);
 
-	qmi_service_unregister_all(data->pds);
+	qmi_service_free(data->pds);
 
-	qmi_service_unref(data->pds);
-
-	g_free(data);
+	l_free(data);
 }
 
 static const struct ofono_location_reporting_driver driver = {
-	.name		= "qmimodem",
 	.type		= OFONO_LOCATION_REPORTING_TYPE_NMEA,
 	.probe		= qmi_location_reporting_probe,
 	.remove		= qmi_location_reporting_remove,
@@ -287,12 +270,4 @@ static const struct ofono_location_reporting_driver driver = {
 	.disable	= qmi_location_reporting_disable,
 };
 
-void qmi_location_reporting_init()
-{
-	ofono_location_reporting_driver_register(&driver);
-}
-
-void qmi_location_reporting_exit()
-{
-	ofono_location_reporting_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(location_reporting, qmimodem, &driver)
