@@ -35,11 +35,12 @@
 #include <ofono/modem.h>
 #include <ofono/gprs-context.h>
 
+#include <drivers/atmodem/atutil.h>
+
 #include "gatchat.h"
 #include "gatresult.h"
 #include "gattty.h"
 
-#include "swmodem.h"
 #include "src/missing.h"
 
 static const char *none_prefix[] = { NULL };
@@ -244,19 +245,10 @@ static void sw_gprs_context_remove(struct ofono_gprs_context *gc)
 }
 
 static const struct ofono_gprs_context_driver driver = {
-	.name			= "swmodem",
 	.probe			= sw_gprs_context_probe,
 	.remove			= sw_gprs_context_remove,
 	.activate_primary	= sw_gprs_activate_primary,
 	.deactivate_primary	= sw_gprs_deactivate_primary,
 };
 
-void sw_gprs_context_init(void)
-{
-	ofono_gprs_context_driver_register(&driver);
-}
-
-void sw_gprs_context_exit(void)
-{
-	ofono_gprs_context_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(gprs_context, swmodem, &driver)
