@@ -91,6 +91,7 @@ static int ublox_probe(struct ofono_modem *modem)
 		return -ENOMEM;
 
 	ofono_modem_set_data(modem, data);
+	ofono_modem_set_capabilities(modem, OFONO_MODEM_CAPABILITY_LTE);
 
 	return 0;
 }
@@ -490,7 +491,6 @@ static void ublox_post_online(struct ofono_modem *modem)
 }
 
 static struct ofono_modem_driver ublox_driver = {
-	.name		= "ublox",
 	.probe		= ublox_probe,
 	.remove		= ublox_remove,
 	.enable		= ublox_enable,
@@ -501,15 +501,4 @@ static struct ofono_modem_driver ublox_driver = {
 	.post_online	= ublox_post_online,
 };
 
-static int ublox_init(void)
-{
-	return ofono_modem_driver_register(&ublox_driver);
-}
-
-static void ublox_exit(void)
-{
-	ofono_modem_driver_unregister(&ublox_driver);
-}
-
-OFONO_PLUGIN_DEFINE(ublox, "u-blox modem driver", VERSION,
-		OFONO_PLUGIN_PRIORITY_DEFAULT, ublox_init, ublox_exit)
+OFONO_MODEM_DRIVER_BUILTIN(ublox, &ublox_driver)
