@@ -43,6 +43,7 @@
 #include "ofono.h"
 
 #include "common.h"
+#include "provisiondb.h"
 #include "storage.h"
 #include "simutil.h"
 #include "util.h"
@@ -176,6 +177,10 @@ static const char *gprs_context_default_name(enum ofono_gprs_context_type type)
 		return "WAP";
 	case OFONO_GPRS_CONTEXT_TYPE_IMS:
 		return "IMS";
+	case OFONO_GPRS_CONTEXT_TYPE_SUPL:
+		return "SUPL";
+	case OFONO_GPRS_CONTEXT_TYPE_IA:
+		return "Initial Attach";
 	}
 
 	return NULL;
@@ -195,6 +200,10 @@ static const char *gprs_context_type_to_string(
 		return "wap";
 	case OFONO_GPRS_CONTEXT_TYPE_IMS:
 		return "ims";
+	case OFONO_GPRS_CONTEXT_TYPE_SUPL:
+		return "supl";
+	case OFONO_GPRS_CONTEXT_TYPE_IA:
+		return "ia";
 	}
 
 	return NULL;
@@ -215,6 +224,9 @@ static gboolean gprs_context_string_to_type(const char *str,
 	} else if (g_str_equal(str, "ims")) {
 		*out = OFONO_GPRS_CONTEXT_TYPE_IMS;
 		return TRUE;
+	} else if (g_str_equal(str, "supl")) {
+		*out = OFONO_GPRS_CONTEXT_TYPE_SUPL;
+		return TRUE;
 	}
 
 	return FALSE;
@@ -313,20 +325,20 @@ static struct pri_context *gprs_context_by_path(struct ofono_gprs *gprs,
 static void context_settings_free(struct context_settings *settings)
 {
 	if (settings->ipv4) {
-		g_free(settings->ipv4->ip);
-		g_free(settings->ipv4->netmask);
-		g_free(settings->ipv4->gateway);
-		g_strfreev(settings->ipv4->dns);
-		g_free(settings->ipv4->proxy);
+		l_free(settings->ipv4->ip);
+		l_free(settings->ipv4->netmask);
+		l_free(settings->ipv4->gateway);
+		l_strv_free(settings->ipv4->dns);
+		l_free(settings->ipv4->proxy);
 
 		g_free(settings->ipv4);
 		settings->ipv4 = NULL;
 	}
 
 	if (settings->ipv6) {
-		g_free(settings->ipv6->ip);
-		g_free(settings->ipv6->gateway);
-		g_strfreev(settings->ipv6->dns);
+		l_free(settings->ipv6->ip);
+		l_free(settings->ipv6->gateway);
+		l_strv_free(settings->ipv6->dns);
 
 		g_free(settings->ipv6);
 		settings->ipv6 = NULL;
@@ -532,7 +544,7 @@ static void pri_parse_proxy(struct pri_context *ctx, const char *proxy)
 {
 	char *scheme, *host, *port, *path;
 
-	scheme = g_strdup(proxy);
+	scheme = l_strdup(proxy);
 	if (scheme == NULL)
 		return;
 
@@ -546,7 +558,7 @@ static void pri_parse_proxy(struct pri_context *ctx, const char *proxy)
 		else if (strcasecmp(scheme, "http") == 0)
 			ctx->proxy_port = 80;
 		else {
-			g_free(scheme);
+			l_free(scheme);
 			return;
 		}
 	} else {
@@ -569,10 +581,10 @@ static void pri_parse_proxy(struct pri_context *ctx, const char *proxy)
 		}
 	}
 
-	g_free(ctx->proxy_host);
-	ctx->proxy_host = g_strdup(host);
+	l_free(ctx->proxy_host);
+	ctx->proxy_host = l_strdup(host);
 
-	g_free(scheme);
+	l_free(scheme);
 }
 
 static void pri_ifupdown(const char *interface, ofono_bool_t active)
@@ -715,7 +727,7 @@ static void pri_reset_context_settings(struct pri_context *ctx)
 	if (ctx->type == OFONO_GPRS_CONTEXT_TYPE_MMS) {
 		pri_set_ipv4_addr(interface, NULL);
 
-		g_free(ctx->proxy_host);
+		l_free(ctx->proxy_host);
 		ctx->proxy_host = NULL;
 		ctx->proxy_port = 0;
 	}
@@ -728,7 +740,7 @@ static void pri_update_mms_context_settings(struct pri_context *ctx)
 	struct ofono_gprs_context *gc = ctx->context_driver;
 	struct context_settings *settings = gc->settings;
 
-	settings->ipv4->proxy = g_strdup(ctx->message_proxy);
+	settings->ipv4->proxy = l_strdup(ctx->message_proxy);
 	pri_parse_proxy(ctx, ctx->message_proxy);
 
 	DBG("proxy %s port %u", ctx->proxy_host, ctx->proxy_port);
@@ -1075,7 +1087,7 @@ static DBusMessage *pri_set_proto(struct pri_context *ctx,
 	GKeyFile *settings = ctx->gprs->settings;
 	enum ofono_gprs_proto proto;
 
-	if (gprs_proto_from_string(str, &proto) == FALSE)
+	if (!gprs_proto_from_string(str, &proto))
 		return __ofono_error_invalid_format(msg);
 
 	if (ctx->context.proto == proto)
@@ -1189,7 +1201,7 @@ static DBusMessage *pri_set_auth_method(struct pri_context *ctx,
 	GKeyFile *settings = ctx->gprs->settings;
 	enum ofono_gprs_auth_method auth;
 
-	if (gprs_auth_method_from_string(str, &auth) == FALSE)
+	if (!gprs_auth_method_from_string(str, &auth))
 		return __ofono_error_invalid_format(msg);
 
 	if (ctx->context.auth_method == auth)
@@ -1398,8 +1410,8 @@ static void pri_context_destroy(gpointer userdata)
 {
 	struct pri_context *ctx = userdata;
 
-	g_free(ctx->proxy_host);
-	g_free(ctx->path);
+	l_free(ctx->proxy_host);
+	l_free(ctx->path);
 	g_free(ctx);
 }
 
@@ -1424,7 +1436,7 @@ static gboolean context_dbus_register(struct pri_context *ctx)
 		return FALSE;
 	}
 
-	ctx->path = g_strdup(path);
+	ctx->path = l_strdup(path);
 	ctx->key = ctx->path + strlen(basepath) + 1;
 
 	return TRUE;
@@ -1956,7 +1968,7 @@ static struct pri_context *find_usable_context(struct ofono_gprs *gprs,
 		 * web.provider.com but it apepars as
 		 * web.provider.com.mncX.mccY.gprs .
 		 */
-		if (g_str_has_prefix(apn, pri_ctx->context.apn))
+		if (l_str_has_prefix(apn, pri_ctx->context.apn))
 			return pri_ctx;
 	}
 
@@ -2181,7 +2193,7 @@ static void gprs_deactivate_for_remove(const struct ofono_error *error,
 	}
 
 	/* Make a backup copy of path for signal emission below */
-	path = g_strdup(ctx->path);
+	path = l_strdup(ctx->path);
 
 	context_dbus_unregister(ctx);
 	gprs->contexts = g_slist_remove(gprs->contexts, ctx);
@@ -2193,7 +2205,7 @@ static void gprs_deactivate_for_remove(const struct ofono_error *error,
 	g_dbus_emit_signal(conn, atompath, OFONO_CONNECTION_MANAGER_INTERFACE,
 				"ContextRemoved", DBUS_TYPE_OBJECT_PATH, &path,
 				DBUS_TYPE_INVALID);
-	g_free(path);
+	l_free(path);
 }
 
 static DBusMessage *gprs_remove_context(DBusConnection *conn,
@@ -2371,11 +2383,12 @@ static DBusMessage *gprs_get_contexts(DBusConnection *conn,
 	return reply;
 }
 
-static void provision_context(const struct ofono_gprs_provision_data *ap,
+static void provision_context(const struct provision_db_entry *ap,
 				struct ofono_gprs *gprs)
 {
 	unsigned int id;
 	struct pri_context *context = NULL;
+	enum ofono_gprs_context_type type;
 
 	/* Sanity check */
 	if (ap == NULL)
@@ -2408,10 +2421,19 @@ static void provision_context(const struct ofono_gprs_provision_data *ap,
 							gprs->last_context_id);
 	else
 		id = l_uintset_find_unused_min(gprs->used_pids);
+
 	if (id > l_uintset_get_max(gprs->used_pids))
 		return;
 
-	context = pri_context_create(gprs, ap->name, ap->type);
+	/*
+	 * Right now oFono is not setup to handle contexts with multiple
+	 * types due to the way the D-Bus API is structured.  This mainly
+	 * affects Internet + MMS contexts.  For now, work around this
+	 * by selecting the primary (lowest bit) context type
+	 */
+	type = 1 << (__builtin_ffs(ap->type) - 1);
+
+	context = pri_context_create(gprs, ap->name, type);
 	if (context == NULL)
 		return;
 
@@ -2429,7 +2451,7 @@ static void provision_context(const struct ofono_gprs_provision_data *ap,
 	strcpy(context->context.apn, ap->apn);
 	context->context.proto = ap->proto;
 
-	if (ap->type == OFONO_GPRS_CONTEXT_TYPE_MMS) {
+	if (type == OFONO_GPRS_CONTEXT_TYPE_MMS) {
 		if (ap->message_proxy != NULL)
 			strcpy(context->message_proxy, ap->message_proxy);
 
@@ -2453,12 +2475,11 @@ static void provision_context(const struct ofono_gprs_provision_data *ap,
 static void provision_contexts(struct ofono_gprs *gprs, const char *mcc,
 				const char *mnc, const char *spn)
 {
-	struct ofono_gprs_provision_data *settings;
-	int count;
-	int i;
+	struct provision_db_entry *settings;
+	size_t count;
+	size_t i;
 
-	if (__ofono_gprs_provision_get_settings(mcc, mnc, spn,
-						&settings, &count) == FALSE) {
+	if (!__ofono_provision_get_settings(mcc, mnc, spn, &settings, &count)) {
 		ofono_warn("Provisioning failed");
 		return;
 	}
@@ -2466,7 +2487,7 @@ static void provision_contexts(struct ofono_gprs *gprs, const char *mcc,
 	for (i = 0; i < count; i++)
 		provision_context(&settings[i], gprs);
 
-	__ofono_gprs_provision_free_settings(settings, count);
+	l_free(settings);
 }
 
 static void remove_non_active_context(struct ofono_gprs *gprs,
@@ -2481,7 +2502,7 @@ static void remove_non_active_context(struct ofono_gprs *gprs,
 	}
 
 	/* Make a backup copy of path for signal emission below */
-	path = g_strdup(ctx->path);
+	path = l_strdup(ctx->path);
 
 	context_dbus_unregister(ctx);
 	gprs->contexts = g_slist_remove(gprs->contexts, ctx);
@@ -2490,7 +2511,7 @@ static void remove_non_active_context(struct ofono_gprs *gprs,
 	g_dbus_emit_signal(conn, atompath, OFONO_CONNECTION_MANAGER_INTERFACE,
 				"ContextRemoved", DBUS_TYPE_OBJECT_PATH, &path,
 				DBUS_TYPE_INVALID);
-	g_free(path);
+	l_free(path);
 }
 
 static DBusMessage *gprs_reset_contexts(DBusConnection *conn,
@@ -2795,7 +2816,7 @@ static void gprs_context_remove(struct ofono_atom *atom)
 	if (gc->driver && gc->driver->remove)
 		gc->driver->remove(gc);
 
-	g_free(gc->interface);
+	l_free(gc->interface);
 	g_free(gc);
 }
 
@@ -2848,8 +2869,8 @@ const char *ofono_gprs_context_get_interface(struct ofono_gprs_context *gc)
 void ofono_gprs_context_set_interface(struct ofono_gprs_context *gc,
 					const char *interface)
 {
-	g_free(gc->interface);
-	gc->interface = g_strdup(interface);
+	l_free(gc->interface);
+	gc->interface = l_strdup(interface);
 }
 
 void ofono_gprs_context_set_ipv4_address(struct ofono_gprs_context *gc,
@@ -2861,8 +2882,8 @@ void ofono_gprs_context_set_ipv4_address(struct ofono_gprs_context *gc,
 	if (settings->ipv4 == NULL)
 		return;
 
-	g_free(settings->ipv4->ip);
-	settings->ipv4->ip = g_strdup(address);
+	l_free(settings->ipv4->ip);
+	settings->ipv4->ip = l_strdup(address);
 	settings->ipv4->static_ip = static_ip;
 }
 
@@ -2874,8 +2895,8 @@ void ofono_gprs_context_set_ipv4_netmask(struct ofono_gprs_context *gc,
 	if (settings->ipv4 == NULL)
 		return;
 
-	g_free(settings->ipv4->netmask);
-	settings->ipv4->netmask = g_strdup(netmask);
+	l_free(settings->ipv4->netmask);
+	settings->ipv4->netmask = l_strdup(netmask);
 }
 
 void ofono_gprs_context_set_ipv4_prefix_length(struct ofono_gprs_context *gc,
@@ -2888,7 +2909,7 @@ void ofono_gprs_context_set_ipv4_prefix_length(struct ofono_gprs_context *gc,
 	if (settings->ipv4 == NULL)
 		return;
 
-	g_free(settings->ipv4->netmask);
+	l_free(settings->ipv4->netmask);
 
 	memset(&ipv4, 0, sizeof(ipv4));
 
@@ -2896,7 +2917,7 @@ void ofono_gprs_context_set_ipv4_prefix_length(struct ofono_gprs_context *gc,
 		ipv4.s_addr = htonl(~((1 << (32 - length)) - 1));
 
 	inet_ntop(AF_INET, &ipv4, buf, sizeof(buf));
-	settings->ipv4->netmask = g_strdup(buf);
+	settings->ipv4->netmask = l_strdup(buf);
 }
 
 void ofono_gprs_context_set_ipv4_gateway(struct ofono_gprs_context *gc,
@@ -2907,8 +2928,8 @@ void ofono_gprs_context_set_ipv4_gateway(struct ofono_gprs_context *gc,
 	if (settings->ipv4 == NULL)
 		return;
 
-	g_free(settings->ipv4->gateway);
-	settings->ipv4->gateway = g_strdup(gateway);
+	l_free(settings->ipv4->gateway);
+	settings->ipv4->gateway = l_strdup(gateway);
 }
 
 void ofono_gprs_context_set_ipv4_dns_servers(struct ofono_gprs_context *gc,
@@ -2919,8 +2940,8 @@ void ofono_gprs_context_set_ipv4_dns_servers(struct ofono_gprs_context *gc,
 	if (settings->ipv4 == NULL)
 		return;
 
-	g_strfreev(settings->ipv4->dns);
-	settings->ipv4->dns = g_strdupv((char **) dns);
+	l_strv_free(settings->ipv4->dns);
+	settings->ipv4->dns = l_strv_copy((char **) dns);
 }
 
 void ofono_gprs_context_set_ipv6_address(struct ofono_gprs_context *gc,
@@ -2931,8 +2952,8 @@ void ofono_gprs_context_set_ipv6_address(struct ofono_gprs_context *gc,
 	if (settings->ipv6 == NULL)
 		return;
 
-	g_free(settings->ipv6->ip);
-	settings->ipv6->ip = g_strdup(address);
+	l_free(settings->ipv6->ip);
+	settings->ipv6->ip = l_strdup(address);
 }
 
 void ofono_gprs_context_set_ipv6_prefix_length(struct ofono_gprs_context *gc,
@@ -2954,8 +2975,8 @@ void ofono_gprs_context_set_ipv6_gateway(struct ofono_gprs_context *gc,
 	if (settings->ipv6 == NULL)
 		return;
 
-	g_free(settings->ipv6->gateway);
-	settings->ipv6->gateway = g_strdup(gateway);
+	l_free(settings->ipv6->gateway);
+	settings->ipv6->gateway = l_strdup(gateway);
 }
 
 void ofono_gprs_context_set_ipv6_dns_servers(struct ofono_gprs_context *gc,
@@ -2966,8 +2987,8 @@ void ofono_gprs_context_set_ipv6_dns_servers(struct ofono_gprs_context *gc,
 	if (settings->ipv6 == NULL)
 		return;
 
-	g_strfreev(settings->ipv6->dns);
-	settings->ipv6->dns = g_strdupv((char **) dns);
+	l_strv_free(settings->ipv6->dns);
+	settings->ipv6->dns = l_strv_copy((char **) dns);
 }
 
 static void free_contexts(struct ofono_gprs *gprs)
@@ -2978,7 +2999,7 @@ static void free_contexts(struct ofono_gprs *gprs)
 		storage_close(gprs->imsi, SETTINGS_STORE,
 				gprs->settings, TRUE);
 
-		g_free(gprs->imsi);
+		l_free(gprs->imsi);
 		gprs->imsi = NULL;
 		gprs->settings = NULL;
 	}
@@ -3141,7 +3162,7 @@ static gboolean load_context(struct ofono_gprs *gprs, const char *group)
 	if (protostr == NULL)
 		protostr = g_strdup("ip");
 
-	if (gprs_proto_from_string(protostr, &proto) == FALSE)
+	if (!gprs_proto_from_string(protostr, &proto))
 		goto error;
 
 	username = g_key_file_get_string(gprs->settings, group,
@@ -3162,7 +3183,7 @@ static gboolean load_context(struct ofono_gprs *gprs, const char *group)
 	if (authstr == NULL)
 		authstr = g_strdup("chap");
 
-	if (gprs_auth_method_from_string(authstr, &auth) == FALSE)
+	if (!gprs_auth_method_from_string(authstr, &auth))
 		goto error;
 
 	if (strlen(password) > OFONO_GPRS_MAX_PASSWORD_LENGTH)
@@ -3245,7 +3266,7 @@ static void gprs_load_settings(struct ofono_gprs *gprs, const char *imsi)
 	if (gprs->settings == NULL)
 		return;
 
-	gprs->imsi = g_strdup(imsi);
+	gprs->imsi = l_strdup(imsi);
 
 	error = NULL;
 	gprs->powered = g_key_file_get_boolean(gprs->settings, SETTINGS_GROUP,
@@ -3283,8 +3304,8 @@ static void gprs_load_settings(struct ofono_gprs *gprs, const char *imsi)
 		if (g_str_equal(groups[i], SETTINGS_GROUP))
 			continue;
 
-		if (!g_str_has_prefix(groups[i], "context")) {
-			if (!g_str_has_prefix(groups[i], "primarycontext"))
+		if (!l_str_has_prefix(groups[i], "context")) {
+			if (!l_str_has_prefix(groups[i], "primarycontext"))
 				goto remove;
 
 			legacy = TRUE;
