@@ -138,7 +138,7 @@ static void sim_auth_unregister(struct ofono_atom *atom)
 	struct ofono_sim_auth *sa = __ofono_atom_get_data(atom);
 
 	free_apps(sa);
-	g_free(sa->nai);
+	l_free(sa->nai);
 
 	if (sa->pending) {
 		__ofono_dbus_pending_reply(&sa->pending->msg,
@@ -207,14 +207,10 @@ static void handle_umts(struct ofono_sim_auth *sa, const uint8_t *resp,
 	DBusMessage *reply = NULL;
 	DBusMessageIter iter;
 	DBusMessageIter dict;
-	const uint8_t *res = NULL;
-	const uint8_t *ck = NULL;
-	const uint8_t *ik = NULL;
-	const uint8_t *auts = NULL;
-	const uint8_t *kc = NULL;
+	struct data_block res, ck, ik, auts, sres, kc;
 
 	if (!sim_parse_umts_authenticate(resp, len, &res, &ck, &ik,
-			&auts, &kc))
+			&auts, &sres, &kc))
 		goto umts_end;
 
 	reply = dbus_message_new_method_return(sa->pending->msg);
@@ -224,15 +220,23 @@ static void handle_umts(struct ofono_sim_auth *sa, const uint8_t *resp,
 	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY,
 			"{say}", &dict);
 
-	if (auts) {
-		append_dict_byte_array(&dict, "AUTS", auts, 14);
-	} else {
-		append_dict_byte_array(&dict, "RES", res, 8);
-		append_dict_byte_array(&dict, "CK", ck, 16);
-		append_dict_byte_array(&dict, "IK", ik, 16);
-		if (kc)
-			append_dict_byte_array(&dict, "Kc", kc, 8);
-	}
+	if (auts.data)
+		append_dict_byte_array(&dict, "AUTS", auts.data, auts.len);
+
+	if (sres.data)
+		append_dict_byte_array(&dict, "SRES", sres.data, sres.len);
+
+	if (res.data)
+		append_dict_byte_array(&dict, "RES", res.data, res.len);
+
+	if (ck.data)
+		append_dict_byte_array(&dict, "CK", ck.data, ck.len);
+
+	if (ik.data)
+		append_dict_byte_array(&dict, "IK", ik.data, ik.len);
+
+	if (kc.data)
+		append_dict_byte_array(&dict, "Kc", kc.data, kc.len);
 
 	dbus_message_iter_close_container(&iter, &dict);
 
@@ -384,7 +388,8 @@ static DBusMessage *usim_gsm_authenticate(DBusConnection *conn,
 	if (sa->pending)
 		return __ofono_error_busy(msg);
 
-	dbus_message_iter_init(msg, &iter);
+	if (!dbus_message_iter_init(msg, &iter))
+		return __ofono_error_invalid_args(msg);
 
 	if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_ARRAY)
 		return __ofono_error_invalid_format(msg);
@@ -667,8 +672,8 @@ static char *build_nai(const char *imsi)
 		memcpy(mnc + 1, imsi + 3, 2);
 	}
 
-	nai = g_strdup_printf("%s@ims.mnc%.3s.mcc%.3s.3gppnetwork.org",
-			imsi, mnc, mcc);
+	nai = l_strdup_printf("%s@ims.mnc%.3s.mcc%.3s.3gppnetwork.org",
+				imsi, mnc, mcc);
 
 	return nai;
 }
@@ -747,7 +752,7 @@ loop_end:
 	if (!__ofono_sim_get_impi(sa->sim))
 		sa->nai = build_nai(ofono_sim_get_imsi(sa->sim));
 	else
-		sa->nai = g_strdup(__ofono_sim_get_impi(sa->sim));
+		sa->nai = l_strdup(__ofono_sim_get_impi(sa->sim));
 
 	g_dbus_register_interface(conn, path,
 			OFONO_SIM_AUTHENTICATION_INTERFACE,
