@@ -161,8 +161,8 @@ void stk_agent_free(struct stk_agent *agent)
 	if (agent->removed_cb)
 		agent->removed_cb(agent->removed_data);
 
-	g_free(agent->path);
-	g_free(agent->bus);
+	l_free(agent->path);
+	l_free(agent->bus);
 	g_free(agent);
 }
 
@@ -236,8 +236,8 @@ struct stk_agent *stk_agent_new(const char *path, const char *sender,
 	if (agent == NULL)
 		return NULL;
 
-	agent->path = g_strdup(path);
-	agent->bus = g_strdup(sender);
+	agent->path = l_strdup(path);
+	agent->bus = l_strdup(sender);
 	agent->remove_on_terminate = remove_on_terminate;
 
 	agent->disconnect_watch = g_dbus_add_disconnect_watch(conn, sender,
@@ -639,6 +639,8 @@ static void get_key_cb(DBusPendingCall *call, void *data)
 	enum stk_agent_result result;
 	bool remove_agent;
 	char *key;
+	int len;
+	wchar_t cp;
 
 	if (check_error(agent, reply,
 			ALLOWED_ERROR_GO_BACK | ALLOWED_ERROR_TERMINATE,
@@ -654,13 +656,19 @@ static void get_key_cb(DBusPendingCall *call, void *data)
 
 	if (dbus_message_get_args(reply, NULL,
 					DBUS_TYPE_STRING, &key,
-					DBUS_TYPE_INVALID) == FALSE ||
-			g_utf8_strlen(key, 10) != 1) {
+					DBUS_TYPE_INVALID) == FALSE) {
 		ofono_error("Can't parse the reply to GetKey()");
 		remove_agent = true;
 		goto error;
 	}
 
+	len = strlen(key);
+	if (l_utf8_get_codepoint(key, len, &cp) != len) {
+		ofono_error("GetKey() return expected a single character");
+		remove_agent = true;
+		goto error;
+	}
+
 	cb(result, key, agent->user_data);
 
 	CALLBACK_END();
@@ -830,7 +838,7 @@ static void get_input_cb(DBusPendingCall *call, void *data)
 		goto error;
 	}
 
-	len = g_utf8_strlen(string, -1);
+	len = l_utf8_strlen(string);
 
 	if (len < agent->min_length || len > agent->max_length) {
 		ofono_error("Length not acceptable");
